unit Main;

 {* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright (c) 2006 by Martin Offenwanger / coder@dsplayer.de            *
  * http://www.dsplayer.com                                                 *
  *                                                                         *
  * This library is free software; you can redistribute it and/or           *
  * modify it under the terms of the GNU Lesser General Public              *
  * License as published by the Free Software Foundation; either            *
  * version 2.1 of the License, or (at your option) any later version.      *
  *                                                                         *
  * This library is distributed in the hope that it will be useful,         *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * Lesser General Public License for more details.                         *
  *                                                                         *
  * You should have received a copy of the GNU Lesser General Public        *
  * License along with this library; if not, write to the Free Software     *
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{
@author(Martin Offenwanger: coder@dsplayer.de)
@created(Jul 17, 2006)
@lastmod(Nov 07, 2006)
@version(0.14 alpha)
}

{$MODE Delphi}

interface

uses
  // Common includes (Lazarus & Windows)
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  ComCtrls,
  // Lazarus spezific includes
  LResources, Buttons,
  // BTMPEG Library
  BTMPEGAudioUtil,
  BTMPEGAudioTables,
  BTMPEGAudioTypes,
  BTMPEGAudioConversionsUtil,
  BTCommonConversionsUtil,
  BTCommonEvents,
  BTID3Util;

type
  TFrmMain = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    BtnApplyWriteID3v1: TButton;
    EditTitle: TEdit;
    EdtAlbum: TEdit;
    EdtYear: TEdit;
    EdtComment: TEdit;
    EdtArtist: TEdit;
    OpenDialog1: TOpenDialog;
    StatusBar1: TStatusBar;
    GroupBox2: TGroupBox;
    Label7: TLabel;
    BtnBrowse: TButton;
    LblCurrentFile: TLabel;
    MnoLog: TMemo;
    GroupBox3: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    LblSize: TLabel;
    LblVersion: TLabel;
    LblLayer: TLabel;
    LblCRC: TLabel;
    LblBitrate: TLabel;
    LblSampleRate: TLabel;
    LblPadding: TLabel;
    LblPrivate: TLabel;
    LblChannelMode: TLabel;
    LblModeExtension: TLabel;
    LblCopyrighted: TLabel;
    LblOriginal: TLabel;
    LblEmphasis: TLabel;
    Label21: TLabel;
    EdtTrack: TEdit;
    ChbEnableWriteMode: TCheckBox;
    CmbGenre: TComboBox;
    Label22: TLabel;
    Label23: TLabel;
    LblFrameLength: TLabel;
    Label24: TLabel;
    LblFrameOffset: TLabel;
    Label25: TLabel;
    LblNbrOfFrames: TLabel;
    Label26: TLabel;
    LblLengthInSeconds: TLabel;
    ProgressBar1: TProgressBar;
    procedure BtnBrowseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BtnApplyWriteID3v1Click(Sender: TObject);
    procedure ChbEnableWriteModeClick(Sender: TObject);
  private
    m_LoadedMp3File: string;
    m_StatusbarText: TLabel;
    procedure SetStatusBarText(f_Text: string);
    procedure OnLogEvent(LogInfo: string);
    procedure OnPosChangedEvent(Position: Integer);
    procedure OnMinChangedEvent(Position: Integer);
    procedure OnMaxChangedEvent(Position: Integer);
    procedure ClearMPEGInfoLabels;
  public
  end;

var
  FrmMain: TFrmMain;

implementation


procedure TFrmMain.OnLogEvent(LogInfo: string);
begin
  MnoLog.Lines.Add(LogInfo);
end;

procedure TFrmMain.OnPosChangedEvent(Position: Integer);
begin
  Application.ProcessMessages;
  ProgressBar1.Position := Position;
end;

procedure TFrmMain.OnMinChangedEvent(Position: Integer);
begin
  ProgressBar1.Min := Position;
end;

procedure TFrmMain.OnMaxChangedEvent(Position: Integer);
begin
  ProgressBar1.Max := Position;
end;

procedure TFrmMain.ClearMPEGInfoLabels;
begin
  LblFrameOffset.Caption := EmptyStr;
  LblSize.Caption := EmptyStr;
  LblVersion.Caption := EmptyStr;
  LblLayer.Caption := EmptyStr;
  LblCRC.Caption := EmptyStr;
  LblBitrate.Caption := EmptyStr;
  LblSampleRate.Caption := EmptyStr;
  LblPadding.Caption := EmptyStr;
  LblPrivate.Caption := EmptyStr;
  LblChannelMode.Caption := EmptyStr;
  LblModeExtension.Caption := EmptyStr;
  LblCopyrighted.Caption := EmptyStr;
  LblOriginal.Caption := EmptyStr;
  LblEmphasis.Caption := EmptyStr;
  LblFrameLength.Caption := EmptyStr;
  LblNbrOfFrames.Caption := EmptyStr;
  LblLengthInSeconds.Caption := EmptyStr;
  CmbGenre.Text := EmptyStr;
  EditTitle.Text := EmptyStr;
  EdtArtist.Text := EmptyStr;
  EdtAlbum.Text := EmptyStr;
  EdtYear.Text := EmptyStr;
  EdtComment.Text := EmptyStr;
  EdtTrack.Text := EmptyStr;
  CmbGenre.Text := EmptyStr;
end;


procedure TFrmMain.SetStatusBarText(f_Text: string);
begin
  m_StatusbarText.Caption := '  Status: ' + f_Text;
  StatusBar1.Refresh;
end;

procedure TFrmMain.BtnBrowseClick(Sender: TObject);
var
  FrameHeader: TBTMPEGAudioProperties;
  FrameHeaderStrings: TBTMPEGAudioPropertyStrings;
  ID3v1xTAG: TID3v1xTAGStrings;
begin
  if not OpenDialog1.Execute then exit;
  MnoLog.Lines.Clear;
  ClearMPEGInfoLabels;
  if not FileExists(OpenDialog1.FileName) then begin
    MnoLog.Lines.Add('File ' + OpenDialog1.FileName +
      ' do not exist or no file was loaded');
    SetStatusBarText('Error Warning');
    exit;
  end;
  SetStatusBarText('In Process');
  // try to get ID3 data
  if GetID3v1xTag(OpenDialog1.FileName, ID3v1xTAG) then begin
    // found ID3 Data -> Paint ID3 Strings to GUI
    EditTitle.Text := ID3v1xTAG.Title;
    EdtArtist.Text := ID3v1xTAG.Artist;
    EdtAlbum.Text := ID3v1xTAG.Album;
    EdtYear.Text := ID3v1xTAG.Year;
    EdtComment.Text := ID3v1xTAG.Comment;
    EdtTrack.Text := ID3v1xTAG.Track;
    CmbGenre.Text := ID3v1xTAG.Genre;
  end else
      MnoLog.Lines.Add('No ID3 Tag have been found');
  if ParseMPEGAudioFile(OpenDialog1.FileName, ParseComplete_CBR_VBR,
    FrameHeader) then begin
    // Get info Strings of the Header
    FrameHeaderStrings := BTMPEGAudPropToMPEGAudPropStr(FrameHeader);
    LblFrameOffset.Caption := FrameHeaderStrings.FrameOffset;
    LblSize.Caption := FrameHeaderStrings.FileSize;
    LblVersion.Caption := FrameHeaderStrings.Version;
    LblLayer.Caption := FrameHeaderStrings.Layer;
    LblCRC.Caption := FrameHeaderStrings.CRCProtected;
    LblBitrate.Caption := FrameHeaderStrings.Bitrate + ' ('
      + FrameHeaderStrings.VBR_CBR + ')';
    LblSampleRate.Caption := FrameHeaderStrings.Samplerate;
    LblPadding.Caption := FrameHeaderStrings.Padding;
    LblPrivate.Caption := FrameHeaderStrings.PrivateBit;
    LblChannelMode.Caption := FrameHeaderStrings.CannelMode;
    LblModeExtension.Caption := FrameHeaderStrings.StereoExtensions;
    LblCopyrighted.Caption := FrameHeaderStrings.CopyRighted;
    LblOriginal.Caption := FrameHeaderStrings.Original;
    LblEmphasis.Caption := FrameHeaderStrings.Emphasis;
    LblFrameLength.caption := FrameHeaderStrings.FrameLength;
    LblNbrOfFrames.Caption := FrameHeaderStrings.NbrOfFrames + ' frames';
    LblLengthInSeconds.Caption := FrameHeaderStrings.LengthInSeconds;
  end else
      MnoLog.Lines.Add('Read ID3 Header failed');
  m_LoadedMp3File := OpenDialog1.FileName;
  LblCurrentFile.Caption := ExtractFileName(m_LoadedMp3File);
  SetStatusBarText('Ready');
end;

procedure TFrmMain.FormCreate(Sender: TObject);
var
  i: Integer;
  Stringlist: TStringlist;
begin
  ProgressBar1.DoubleBuffered := True;;
  m_LoadedMp3File := EmptyStr;
  m_StatusbarText := TLabel.Create(StatusBar1);
  m_StatusbarText.Parent := StatusBar1;
  m_StatusbarText.Left := 4;
  m_StatusbarText.Top := 4;
  m_StatusbarText.Width := StatusBar1.Width - 8;
  m_StatusbarText.Show;
  SetStatusBarText('Ready');
  OnObjLogEvent := OnLogEvent;
  OnObjPosChangedEvent := OnPosChangedEvent;
  OnObjMinChangedEvent := OnMinChangedEvent;
  OnObjMaxChangedEvent := OnMaxChangedEvent;
  ClearMPEGInfoLabels;
  Stringlist := TStringList.Create;
  for i := 0 to High(ID3Genre) - 1 do
    Stringlist.Add(ID3Genre[i]);
  Stringlist.Sort;
  for i := 0 to High(ID3Genre) - 1 do
    CmbGenre.Items.Add(Stringlist.Strings[i]);
  Stringlist.Free;
end;

procedure TFrmMain.BtnApplyWriteID3v1Click(Sender: TObject);
var
  ID3v1TAG: TID3v1TAG;
  ID3v11TAG: TID3v11TAG;
  pID3v1xTag: Pointer;
  i: Byte;
begin
  if not FileExists(m_LoadedMp3File) then begin
    MnoLog.Lines.Add('File ' + m_LoadedMp3File + ' do not exist or no file was loaded');
    SetStatusBarText('Error Warning');
    exit;
  end;
  FillChar(ID3v1TAG,sizeof(TID3v1TAG), $00); // Zero record....
  FillChar(ID3v11TAG,sizeof(TID3v11TAG), $00); // Zero record....
  ID3v1TAG.Signature := ID3v1xTagID;
  ID3v11TAG.Signature := ID3v1xTagID;
  if EdtTrack.Text <> EmptyStr then begin
    try
      // fill id3 version 1.1 record
      if (StrToInt(EdtTrack.Text) > 255) or (StrToInt(EdtTrack.Text) < 1) then
        Abort;
      BTTStringArrayOfChar(EditTitle.Text, ID3v11TAG.Title);
      BTTStringArrayOfChar(EdtArtist.Text, ID3v11TAG.Artist);
      BTTStringArrayOfChar(EdtAlbum.Text, ID3v11TAG.Album);
      BTTStringArrayOfChar(EdtYear.Text, ID3v11TAG.Year);
      BTTStringArrayOfChar(EdtComment.Text, ID3v11TAG.Comment);
      ID3v11TAG.Genre := 127;
      for i := 0 to High(ID3Genre) do
        if CmbGenre.Text = ID3Genre[i] then begin
          ID3v11TAG.Genre := i;
          break;
        end;
      ID3v11TAG.Track := Byte(StrToInt(EdtTrack.Text));
      pID3v1xTag := @ID3v11TAG;
    except
      MnoLog.Lines.Add('Track number must be numerical and in the range of 1 - 255');
      SetStatusBarText('Error Warning');
      exit;
    end;
  end else begin
    // fill id3 version 1.0 record
    BTTStringArrayOfChar(EditTitle.Text, ID3v1TAG.Title);
    BTTStringArrayOfChar(EdtArtist.Text, ID3v1TAG.Artist);
    BTTStringArrayOfChar(EdtAlbum.Text, ID3v1TAG.Album);
    BTTStringArrayOfChar(EdtYear.Text, ID3v1TAG.Year);
    BTTStringArrayOfChar(EdtComment.Text, ID3v1TAG.Comment);
    ID3v1TAG.Genre := 127;
    for i := 0 to High(ID3Genre) do
      if CmbGenre.Text = ID3Genre[i] then begin
        ID3v1TAG.Genre := i;
        break;
      end;
    pID3v1xTag := @ID3v1TAG;
  end;
  if not SetID3v1xTag(pID3v1xTag, m_LoadedMp3File) then begin
    MnoLog.Lines.Add('Write ID3v1.x Tag Failed');
    SetStatusBarText('Error Warning');
  end else begin
    MnoLog.Lines.Add('ID3v1.x Tag successfully written');
    SetStatusBarText('Ready');
  end;
end;

procedure TFrmMain.ChbEnableWriteModeClick(Sender: TObject);
begin
  if ChbEnableWriteMode.Checked then begin
    CmbGenre.Enabled := True;
    EditTitle.Enabled := True;
    EdtArtist.Enabled := True;
    EdtAlbum.Enabled := True;
    EdtYear.Enabled := True;
    EdtComment.Enabled := True;
    EdtTrack.Enabled := True;
    CmbGenre.Enabled := True;
    BtnApplyWriteID3v1.Enabled := True;
  end else begin
    CmbGenre.Enabled := False;
    EditTitle.Enabled := False;
    EdtArtist.Enabled := False;
    EdtAlbum.Enabled := False;
    EdtYear.Enabled := False;
    EdtComment.Enabled := False;
    EdtTrack.Enabled := False;
    CmbGenre.Enabled := False;
    BtnApplyWriteID3v1.Enabled := False;
  end;
end;

initialization
  {$i Main.lrs}

end.

