unit BTCommonEvents;

 {* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright (c) 2006 by Martin Offenwanger / coder@dsplayer.de            *
  * http://www.dsplayer.com                                                 *
  *                                                                         *
  * This library is free software; you can redistribute it and/or           *
  * modify it under the terms of the GNU Lesser General Public              *
  * License as published by the Free Software Foundation; either            *
  * version 2.1 of the License, or (at your option) any later version.      *
  *                                                                         *
  * This library is distributed in the hope that it will be useful,         *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * Lesser General Public License for more details.                         *
  *                                                                         *
  * You should have received a copy of the GNU Lesser General Public        *
  * License along with this library; if not, write to the Free Software     *
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{
@author(Martin Offenwanger: coder@dsplayer.de)
@created(Jul 17, 2006)
@lastmod(Nov 07, 2006)
@version(0.14 alpha)
}

{$IFDEF FPC}
{$MODE Delphi}
{$ENDIF}

interface

(*********************)
(*** Global Events ***)
(*********************)

type
  // "LogEvent" for functions of Object
  TOnObjLogEvent = procedure(LogInfo: string) of object;
  // "LogEvent" for static functions
  TOnLogEvent = procedure(LogInfo: string);

var
  // class event object
  OnObjLogEvent: TOnObjLogEvent = nil;
  // non class event object
  OnLogEvent: TOnLogEvent = nil;
  
procedure RaiseLogEvent(f_logInfo: string);


type
  // "PositionEvent" for functions of Object
  TOnObjFilePosEvent = procedure(Position: Integer) of object;
  // "PositionEvent" for static functions
  TOnFilePosEvent = procedure(Position: Integer);

var
  // class event objects
  OnObjPosChangedEvent: TOnObjFilePosEvent = nil;
  OnObjMinChangedEvent: TOnObjFilePosEvent = nil;
  OnObjMaxChangedEvent: TOnObjFilePosEvent = nil;
  // non class event objects
  OnPosChangedEvent: TOnFilePosEvent = nil;
  OnMinChangedEvent: TOnFilePosEvent = nil;
  OnMaxChangedEvent: TOnFilePosEvent = nil;
  
procedure RaisePosChangedEvent(f_Pos: Integer);
procedure RaiseMinChangedEvent(f_Pos: Integer);
procedure RaiseMaxChangedEvent(f_Pos: Integer);

implementation


procedure RaiseLogEvent(f_logInfo: string);
begin
  if Assigned(OnObjLogEvent) then
    OnObjLogEvent(f_logInfo);
  if Assigned(OnLogEvent) then
    OnObjLogEvent(f_logInfo);
end;

procedure RaisePosChangedEvent(f_Pos: Integer);
begin
  if Assigned(OnObjPosChangedEvent) then
    OnObjPosChangedEvent(f_Pos);
  if Assigned(OnPosChangedEvent) then
    OnPosChangedEvent(f_Pos);
end;

procedure RaiseMinChangedEvent(f_Pos: Integer);
begin
  if Assigned(OnObjMinChangedEvent) then
    OnObjMinChangedEvent(f_Pos);
  if Assigned(OnMinChangedEvent) then
    OnMinChangedEvent(f_Pos);
end;

procedure RaiseMaxChangedEvent(f_Pos: Integer);
begin
  if Assigned(OnObjMaxChangedEvent) then
    OnObjMaxChangedEvent(f_Pos);
  if Assigned(OnMaxChangedEvent) then
    OnMaxChangedEvent(f_Pos);
end;

end.

