unit BTID3Util;

 {* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright (c) 2006 by Martin Offenwanger / coder@dsplayer.de            *
  * http://www.dsplayer.com                                                 *
  *                                                                         *
  * This library is free software; you can redistribute it and/or           *
  * modify it under the terms of the GNU Lesser General Public              *
  * License as published by the Free Software Foundation; either            *
  * version 2.1 of the License, or (at your option) any later version.      *
  *                                                                         *
  * This library is distributed in the hope that it will be useful,         *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * Lesser General Public License for more details.                         *
  *                                                                         *
  * You should have received a copy of the GNU Lesser General Public        *
  * License along with this library; if not, write to the Free Software     *
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{
@author(Martin Offenwanger: coder@dsplayer.de)
@created(Jul 17, 2006)
@lastmod(Nov 07, 2006)
@version(0.14 alpha)
}

////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
//  About this UNIT:
//  ----------------
//  This implementation is about reading and writing ID3 TAGs from and to
//  MPEG Audio Files.
//
//  => Supported ID3 Versions are: ID3v1.0 and ID3v1.1 <=
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////

{$IFDEF FPC}
{$MODE Delphi}
{$ENDIF}

interface

uses SysUtils, Classes;

const

(*************************
 *** Genre LookupTable ***
 *************************)

  // Here's the genre (Max. 256 entries).
  ID3Genre: array[0..125] of string = ('Blues', 'Classic Rock',
    'Country', 'Dance', 'Disco', 'Funk', 'Grunge', 'Hip-Hop',
    'Jazz', 'Metal', 'New Age', 'Oldies', 'Other', 'Pop', 'R&B',
    'Rap', 'Reggae', 'Rock', 'Techno', 'Industrial', 'Alternative',
    'Ska', 'Death Metal', 'Pranks', 'Soundtrack', 'Euro-Techno',
    'Ambient', 'Trip-Hop', 'Vocal', 'Jazz+Funk', 'Fusion', 'Trance',
    'Classical', 'Instrumental', 'Acid', 'House', 'Game', 'Sound Clip',
    'Gospel', 'Noise', 'AlternRock', 'Bass', 'Soul', 'Punk',
    'Space', 'Meditative', 'Instrumental Pop', 'Instrumental Rock',
    'Ethnic', 'Gothic', 'Darkwave', 'Techno-Industrial', 'Electronic',
    'Pop-Folk', 'Eurodance', 'Dream', 'Southern Rock', 'Comedy',
    'Cult', 'Gangsta', 'Top 40', 'Christian Rap', 'Pop/Funk',
    'Jungle', 'Native American', 'Cabaret', 'New Wave', 'Psychadelic',
    'Rave', 'Showtunes', 'Trailer', 'Lo-Fi', 'Tribal', 'Acid Punk',
    'Acid Jazz', 'Polka', 'Retro', 'Musical', 'Rock & Roll',
    'Hard Rock', 'Folk', 'Folk-Rock', 'National Folk', 'Swing',
    'Fast Fusion', 'Bebob', 'Latin', 'Revival', 'Celtic', 'Bluegrass',
    'Avantgarde', 'Gothic Rock', 'Progressive Rock', 'Psychedelic Rock',
    'Symphonic Rock', 'Slow Rock', 'Big Band', 'Chorus', 'Easy Listening',
    'Acoustic', 'Humour', 'Speech', 'Chanson', 'Opera', 'Chamber Music',
    'Sonata', 'Symphony', 'Booty Bass', 'Primus', 'Porn Groove',
    'Satire', 'Slow Jam', 'Club', 'Tango', 'Samba', 'Folklore',
    'Ballad', 'Power Ballad', 'Rhythmic Soul', 'Freestyle', 'Duet',
    'Punk Rock', 'Drum Solo', 'Acapella', 'Euro-House', 'Dance Hall');

(************************)
(*** ID3 Signature/ID ***)
(************************)

   ID3v1xTagID = 'TAG';


(******************************)
(*** ID3 Version Defnitions ***)
(******************************)
type
  TBTDynamicByteArray = array of Byte;
  TID3Version = Byte;
const
  ID3Version10 = 0;
  ID3Version11 = 1;
  ID3VersionStrings: array[0..1] of string = (('ID3v1.0'), ('ID3v1.1'));
  ID3v1xSize = 128;


(***********************************************)
(*** The MPEG Layer 3 ID3v1.x tag structures ***)
(***********************************************)

type
  (* ID3v1.x - Eeasy to use struct
     -> simply combines v1.0 and v1.1 into one struct of strings <-
  *)
  TID3v1xTAGStrings = record
    Title: string;
    Artist: string;
    Album: string;
    Year: string;
    Comment: string;
    Track: string;
    Genre: string;
  end;
  PID3v1xTAGStrings = ^TID3v1xTAGStrings;

  // ID3v1.0
  TID3v1TAG = record
    Signature: array[0..2] of Char; // Valid Signature is: 'TAG'
    Title: array[0..29] of Char;
    Artist: array[0..29] of Char;
    Album: array[0..29] of Char;
    Year: array[0..3] of Char;
    Comment: array[0..29] of Char;
    Genre: Byte;
  end;
  PID3v1TAG = ^TID3v1TAG;

  // ID3v1.1
  TID3v11TAG = record
    Signature: array[0..2] of Char; // Valid Signature is: 'TAG'
    Title: array[0..29] of Char;
    Artist: array[0..29] of Char;
    Album: array[0..29] of Char;
    Year: array[0..3] of Char;
    Comment: array[0..28] of Char;
    Track: Byte;
    Genre: Byte;
  end;
  PID3v11TAG = ^TID3v11TAG;
  

(* ID3 Version Check *)
function GetID3Version(f_Filename: string;
  out ID3Version: TID3Version): Boolean;

(* Read ID3v1.x TAG *)
function GetID3v1xTag(f_Filename: string;
  out f_ID3v1xTAG: TID3v1xTAGStrings): Boolean; overload;

(* Write ID3v1.x TAG to File *)
function SetID3v1xTag(fp_ID31xTAG: TID3v1xTAGStrings;
  f_FileName: string): Boolean; overload;


(* Read ID3v1.0 or ID3v1.1 TAG *)
function GetID3v1xTag(f_Filename: string; out fp_ID31TAG: PID3v1TAG;
  out fp_ID311TAG: PID3v11TAG): Boolean; overload;

(* Write ID3v1.x TAG to File *)
function SetID3v1xTag(fp_ID31xTAG: Pointer;
  f_FileName: string): Boolean; overload;

(* fp_DestBuffer : - must be assigned (128kb)
                   - Contains ID3 Tag data if suntion return True *)
function GetID3v1xTag(var f_Filename: string;
  out f_ID3TAGBuffer: TBTDynamicByteArray): Boolean; overload;


implementation

function GetID3Version(f_Filename: string;
  out ID3Version: TID3Version): Boolean;
var
  ID3TAGBuffer: TBTDynamicByteArray;
begin
  Result := False;
  if not GetID3v1xTag(f_Filename, ID3TAGBuffer) then
    exit;
  // check if 0 is present (byte 0 is always end of string)
  if Byte(PID3v1TAG(@ID3TAGBuffer[0]).Comment[28]) = Byte(0) then
    // check if a title number is present (byte <> 0)
    if Byte(PID3v1TAG(@ID3TAGBuffer[0]).Comment[29]) <> Byte(0) then begin
      ID3Version := 1; // version 1.1 found
      Result := True;
    end else begin
      ID3Version := 0; // version 1.0 found
      Result := True;
    end;
end;

function GetID3v1xTag(f_Filename: string; out fp_ID31TAG: PID3v1TAG;
  out fp_ID311TAG: PID3v11TAG): Boolean;
var
  ID3TAGBuffer: TBTDynamicByteArray;
begin
  Result := False;
  if (fp_ID311TAG = nil) or (fp_ID31TAG = nil) then
    exit;
  if not GetID3v1xTag(f_Filename, ID3TAGBuffer) then
    exit;
  // check if 0 is present (byte 0 is always end of string)
  if Byte(PID3v1TAG(@ID3TAGBuffer[0]).Comment[28]) = Byte(0) then
    // check if a title number is present (byte <> 0)
    if Byte(PID3v1TAG(@ID3TAGBuffer[0]).Comment[29]) <> Byte(0) then begin
      fp_ID31TAG := nil;
      Move(ID3TAGBuffer[0], fp_ID311TAG^, SizeOf(TID3v11TAG));
      Result := True;
    end else begin
      fp_ID311TAG := nil;
      Move(ID3TAGBuffer[0], fp_ID31TAG^, SizeOf(TID3v1TAG));
      Result := True;
    end;
end;

function SetID3v1xTag(fp_ID31xTAG: TID3v1xTAGStrings;
  f_FileName: string): Boolean; overload;
begin
  Result := False;
end;

function SetID3v1xTag(fp_ID31xTAG: Pointer; f_FileName: string): Boolean;
var
  FileStream: TFileStream;
  ID3TAG: TID3v1TAG;
begin
  Result := True;
  if not FileExists(f_Filename) or (fp_ID31xTAG = nil) then begin
    Result := False;
    exit;
  end;
  // sticky: during writing we allow no other process writing to the file
  FileStream := TFileStream.Create(f_Filename, fmOpenReadWrite or
    fmShareDenyWrite);
  try
    FileStream.Position := FileStream.Size - ID3v1xSize;
    FileStream.Read(ID3TAG, ID3v1xSize);
    if PID3v1TAG(fp_ID31xTAG).Signature = ID3v1xTagID then
      FileStream.Position := FileStream.Size - ID3v1xSize;
    FileStream.Write(fp_ID31xTAG^, ID3v1xSize);
  except
    Result := False;
  end;
  if FileStream <> nil then
    FileStream.Free;
end;

function GetID3v1xTag(var f_Filename: string;
  out f_ID3TAGBuffer: TBTDynamicByteArray): Boolean;
var
  (* However, prefer TStream based class to access a file *)
  FileStream: TFileStream;
begin
  Result := False;
  if not FileExists(f_Filename) then
    exit;
  Setlength(f_ID3TAGBuffer, ID3v1xSize);
  FileStream := TFileStream.Create(f_Filename, fmOpenRead or fmShareDenyNone);
  FileStream.Position := FileStream.Size - ID3v1xSize;
  FileStream.Read(f_ID3TAGBuffer[0], ID3v1xSize);
  // Check Signature ID2v1.x
  if UpperCase(PID3v1TAG(@f_ID3TAGBuffer[0]).Signature) = ID3v1xTagID then
    Result := True; // falid IDv1.x TAG have been found
  if FileStream <> nil then
    FileStream.Free;
  if Result = false then
    Setlength(f_ID3TAGBuffer, 0);
end;

function GetID3v1xTag(f_Filename: string;
  out f_ID3v1xTAG: TID3v1xTAGStrings): Boolean;
var
  ID3v1TAG: TID3v1TAG;
  ID3v11TAG: TID3v11TAG;
  lpID3v1TAG: PID3v1TAG;
  lpID3v11TAG: PID3v11TAG;
begin
  Result := false;
  if not FileExists(f_Filename) then
    exit;
  FillChar(f_ID3v1xTAG,sizeof(TID3v1xTAGStrings), $00);
  FillChar(ID3v1TAG,sizeof(TID3v1TAG), $00);
  FillChar(ID3v11TAG,sizeof(TID3v1TAG), $00);
  lpID3v1TAG := @ID3v1TAG;
  lpID3v11TAG := @ID3v11TAG;
  if GetID3v1xTag(f_Filename, lpID3v1TAG, lpID3v11TAG) then begin
    if (lpID3v1TAG = nil) and (lpID3v11TAG = nil) then
      exit;
    if lpID3v1TAG <> nil then begin
      // reading id3v1.0
      f_ID3v1xTAG.Title := lpID3v1TAG.Title;
      f_ID3v1xTAG.Artist := lpID3v1TAG.Artist;
      f_ID3v1xTAG.Album := lpID3v1TAG.Album;
      f_ID3v1xTAG.Year := lpID3v1TAG.Year;
      f_ID3v1xTAG.Comment := lpID3v1TAG.Comment;
      f_ID3v1xTAG.Track := EmptyStr;
      if lpID3v1TAG.Genre <= High(ID3Genre) then
        f_ID3v1xTAG.Genre := ID3Genre[lpID3v1TAG.Genre];
    end;
    if lpID3v11TAG <> nil then begin
      // reading id3v1.1
      f_ID3v1xTAG.Artist := lpID3v11TAG.Artist;
      f_ID3v1xTAG.Title := lpID3v11TAG.Title;
      f_ID3v1xTAG.Album := lpID3v11TAG.Album;
      f_ID3v1xTAG.Year := lpID3v11TAG.Year;
      f_ID3v1xTAG.Comment := lpID3v11TAG.Comment;
      f_ID3v1xTAG.Track := IntToStr(lpID3v11TAG.Track);
      if lpID3v11TAG.Genre <= High(ID3Genre) then
        f_ID3v1xTAG.Genre := ID3Genre[lpID3v11TAG.Genre];
    end;
  end else
    exit;
  Result := True;
end;


end.
