unit BTMPEGAudioTables;

 {* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright (c) 2006 by Martin Offenwanger / coder@dsplayer.de            *
  * http://www.dsplayer.com                                                 *
  *                                                                         *
  * This library is free software; you can redistribute it and/or           *
  * modify it under the terms of the GNU Lesser General Public              *
  * License as published by the Free Software Foundation; either            *
  * version 2.1 of the License, or (at your option) any later version.      *
  *                                                                         *
  * This library is distributed in the hope that it will be useful,         *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * Lesser General Public License for more details.                         *
  *                                                                         *
  * You should have received a copy of the GNU Lesser General Public        *
  * License along with this library; if not, write to the Free Software     *
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{
@author(Martin Offenwanger: coder@dsplayer.de)
@created(Jul 17, 2006)
@lastmod(Nov 07, 2006)
@version(0.14 alpha)
}

{$IFDEF FPC}
{$MODE Delphi}
{$ENDIF}

interface

const

////////////////////////////////////////////////////////////////////////////////
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//------------------------------------------------------------------------------
//  MPEGAudio LookUp Tables are mainly used for
//  "TMP3FrameHeaderInfo to TMP3FrameHeaderInfoStrings" conversion   => BEGIN <=
//------------------------------------------------------------------------------
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  MPEGAudioVersionStrings: array[0..3] of string =
  ('reserved',
    'MPEG Version 1 (ISO/IEC 11172-3)',
    'MPEG Version 2 (ISO/IEC 13818-3)',
    'MPEG Version 2.5');

  MPEGAudioLayerStrings: array[1..3] of string =
  (('Layer I'), ('Layer II'), ('Layer III'));

  //////////////////////////////////////////////////////////////////////////////
  //                          => !!! Attention !!! <=
  //----------------------------------------------------------------------------
  //  If protected by CRC, then 2 bytes follows follows the header
  //  before the playliad begins (the 2 btyes are a crc checksum)
  //----------------------------------------------------------------------------
  MPEGAudioCRCProtectedStrings: array[0..1] of string =
  ('Not CRC protected', 'Protected by CRC (16bit CRC follows header)');
  //----------------------------------------------------------------------------
  //////////////////////////////////////////////////////////////////////////////

  MPEGAudioPaddingBitStrings: array[0..1] of string =
  ('Frame is not padded', 'Frame is padded with one extra slot');
  
  MPEGAudioPrivateBitStrings: array[0..1] of string =
  ('Private bit is not set', 'Private bit is set');
  
  MPEGAudioChannelModeStrings: array[0..3] of string =
  ('Stereo', 'Joint stereo (Stereo)',
   'Dual channel (Stereo)',
   'Single channel (Mono)');
   
  //////////////////////////////////////////////////////////////////////////////
  //                          => !!! Attention !!! <=
  //----------------------------------------------------------------------------
  //  MPEGAudioLayer12ModeExStrings: Use only if Audio File using Layer I or II
  //                                 and Channel Mode is Joint stereo.
  //----------------------------------------------------------------------------
  MPEGAudioLayer12ModeExStrings: array[0..3] of string =
  ('bands 4 to 31', 'bands 8 to 31', 'bands 12 to 31', 'bands 16 to 31');
  //----------------------------------------------------------------------------
  //////////////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////////////////////////////////////
  //                          => !!! Attention !!! <=
  //----------------------------------------------------------------------------
  //  MPEGAudioLayer12ModeExStrings: Use only if Audio File using Layer III
  //                                 and Channel Mode is Joint stereo.
  MPEGAudioLayer3ModeExStrings: array[0..3] of string =
  ('Itensity stereo: off / MS stereo: off',
   'Itensity stereo: on / MS stereo: off',
   'Itensity stereo: off / MS stereo: on',
   'Itensity stereo: on / MS stereo: on');
  //----------------------------------------------------------------------------
  //////////////////////////////////////////////////////////////////////////////

  MPEGAudioCopyRightStrings: array[0..1] of string =
  ('Audio is not copyrighted', 'Audio is not copyrighted');
  
  MPEGAudioOriginalStrings: array[0..1] of string =
  ('Copy of original Media', 'Original Media');
  
  MPEGAudioEmphasisStrings: array[0..3] of string =
  ('none', '50/15 ms', 'reserved', 'CCIT J.17');

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//------------------------------------------------------------------------------
//  MPEGAudio LookUp Tables are mainly used for
//  "TMP3FrameHeaderInfo to TMP3FrameHeaderInfoStrings" conversion     => END <=
//------------------------------------------------------------------------------
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
////////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-----------------------------------------------------------------------------
//  MPEGAudio LookUp Tables for
//  "TMP3BinaryFrameHeader to TMP3FrameHeaderInfo" conversion  => Begin <=
//-----------------------------------------------------------------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


(*************************************
 *** MPEGAudio (MP3) LookUp Tables ***
 *************************************)

  // MPEG Version -> if version is 'reserved' then header is invalid
  MPEGAudioVersionInvalidIndex = 1;
  (* Return Values of the Table MPEGAudioVersion:
     - 3 -> MPEG Audio Version 2.5
     - 0 -> invalid Header
     - 2 -> MPEG Audio Version 2
     - 1 -> MPEG Audio Version 1 *)
  MPEGAudioVersion: array[0..3] of Byte = (3, 0, 2, 1);

  // MPEG Layer -> 0 Header is invalid
  MPEGAudioLayerInvalidIndex = 0;
  MPEGAudioLayer: array[1..3] of Byte = (3, 2, 1);

  // CRC Protection
  MPEGAudioCRCProtected: array[0..1] of Boolean = (True, False);


  (* Table "MPEGAudioBitRates" BitRates (kbps)

     Index Meanings:
        MPEGAudioBitRates[Version, Layer, MPEG Audio Header EEEE Bits]
     Index Values:
        Version:  - 1 -> MPEG Audio Version 1
                  - 2 -> MPEG Audio Version 2
                  - 3 -> MPEG Audio Version 2.5
        Layer:    - 1 -> Layer 1
                  - 2 -> Layer 2
                  - 3 -> Layer 3
        MPEG Audio Header EEEE Bits: - get this value from the Header *)
  MPEGAudioBitRatesInvalidHeaderEEEEBits = 15;
  MPEGAudioBitRates: array[1..3] of array[1..3] of array[0..15] of Integer =
  (* Version 1, Layer I *)
  (((0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, -1),
  (* Version 1, Layer II *)
    (0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, -1),
  (* Version 1, Layer III *)
    (0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, -1)),
  { Version 2, Layer I }
    ((0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, -1),
  { Version 2, Layer II }
    (0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, -1),
  { Version 2, Layer III }
    (0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, -1)),
  { Version 2.5, Layer I }
    ((0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, -1),
  { Version 2.5, Layer II }
    (0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, -1),
  { Version 2.5, Layer III }
    (0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, -1)));

  (* Table "MPEGAudioSampleRates" SampleRates (Hz)

     Index Meanings:
        MPEGAudioSampleRates[Version, MPEG Audio Header FF Bits]
     Index Values:
        Version:  - 1 -> MPEG Audio Version 1
                  - 2 -> MPEG Audio Version 2
                  - 3 -> MPEG Audio Version 2.5
        MPEG Audio Header FF Bits: - get this value from the Header *)
  MPEGAudioSampleRatesInvalidHeaderFFBits = 3;
  MPEGAudioSampleRates: array[1..3] of array[0..3] of Integer =
  ((44100, 48000, 32000, -1),
    (22050, 24000, 16000, -1),
    (11025, 12000, 8000, -1));

  (*  Channel Mode *)
  MPEGAudioChannelModeStereo = 0;
  MPEGAudioChannelModeJointStereo = 1;
  MPEGAudioChannelDualStereo = 2;
  MPEGAudioChannelMono = 3;

  (*  Table "MPEGAudioLayer12ModeExStrings" Mode extension for Layer I & II
      (Only if Joint stereo)

      !! Use only if Audio File using Layer I or II and
         Channel Mode is Joint stereo !!

        MPEG Audio Header JJ Bits: - get this value from the Header *)
  MPEGAudioLayer12ModeEx: array[0..3] of Byte = (4, 8, 12, 16);
  MPEGAudioLayer12ModeExHighBand = 31;

  (*  Table "MPEGAudioEmphasisStrings" Copyright Bit

      Index Meanings:
        MPEGAudioEmphasisStrings[MPEG Audio Header MM Bits]
      Index Values:
        MPEG Audio Header MM Bit: - get this value from the Header *)
  MPEGAudioEmphasisInvalidIndex = 2;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-----------------------------------------------------------------------------
//  MPEGAudio LookUp Tables for
//  "TMP3BinaryFrameHeader to TMP3FrameHeaderInfo" conversion  => End <=
//-----------------------------------------------------------------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
///////////////////////////////////////////////////////////////////////////////



implementation

end.
