unit BTMPEGAudioUtil;


 {* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright (c) 2006 by Martin Offenwanger / coder@dsplayer.de            *
  * http://www.dsplayer.com                                                 *
  *                                                                         *
  * This library is free software; you can redistribute it and/or           *
  * modify it under the terms of the GNU Lesser General Public              *
  * License as published by the Free Software Foundation; either            *
  * version 2.1 of the License, or (at your option) any later version.      *
  *                                                                         *
  * This library is distributed in the hope that it will be useful,         *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * Lesser General Public License for more details.                         *
  *                                                                         *
  * You should have received a copy of the GNU Lesser General Public        *
  * License along with this library; if not, write to the Free Software     *
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{
@author(Martin Offenwanger: coder@dsplayer.de)
@created(Jul 17, 2006)
@lastmod(Nov 07, 2006)
@version(0.14 alpha)
}

////////////////////////////////////////////////////////////////////////////////
// This code was created to provide the ability to parse MPEG Audio Files
// with Object Pascal. The base knowledge for creating such a code by
// yourself and a alternative implementation can be found for free at:
// => http://www.dv.co.yu/mpgscript/mpgtools.htm <=
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Focus: The Focus of the current developement state is functionality, but
// ****** the code sould fast enougth for the most use cases ;)
////////////////////////////////////////////////////////////////////////////////

{$IFDEF FPC}
{$MODE Delphi}
{$ENDIF}

(***************)
(*** Defines ***)
(***************)

{$DEFINE LogEvents} // Output LogEvents
{$DEFINE FileActionEvents} // Raise FileActionEvents

interface

uses
  SysUtils, Classes,
  // BT Library
  BTMPEGAudioTables,
  BTMPEGAudioTypes,
  BTMPEGAudioConversionsUtil,
  BTCommonEvents,
  BTID3Util;


////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
// "ParseCompleteMPEGAudioFile"
//
//  TMPEGParseConfig
//  -----------------
//  ParseSingleFrameApprox:
//  ParseMultiFrameApprox:
//  ParseComplete:
//                                                                   => BEGIN <=
//------------------------------------------------------------------------------
function ParseMPEGAudioFile(f_Filename: string; f_ParserConfig: TMPEGParseConfig;
  out f_MPEGAudioProperties: TBTMPEGAudioProperties): Boolean;
//------------------------------------------------------------------------------
// "ParseCompleteMPEGAudioFile"
//                                                                     => END <=
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
// "IsValidMPEGAudioHeader"
// Validation check for illegal Header bits                          => BEGIN <=
//------------------------------------------------------------------------------
function IsValidMPEGAudioHeader(
  f_MPEGAudioBinaryFrameHeader: TBTMPEGAudioBinaryFrameHeader): Boolean;
//------------------------------------------------------------------------------
// "IsValidMPEGAudioHeader"
// Validation check for illegal Header bits                            => END <=
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
// "ParseSingleMPEGAudioHeader" parses a single MPEG Audio Header    => BEGIN <=
//------------------------------------------------------------------------------
//  Common Limitations:
//     - no validation check for illegal Header bits, so the function
//       may fail with infalid Headers
//          => You should run IsValidMPEGAudioHeader check before calling
//             ParseSingleMPEGAudioHeader function
//  TMP3FrameHeaderInfo Limitations:
//     - fills not the FrameOffSet field
//     - fills not the FileSize field
//     - fills not the NbrOfFrames field
//     - fills not the LengthInSeconds field
//     - fills not the VariableBitRate field
//------------------------------------------------------------------------------
function ParseSingleMPEGAudioHeader(
  f_MPEGAudioBinaryFrameHeader: TBTMPEGAudioBinaryFrameHeader;
  out f_MPEGAudioProperties: TBTMPEGAudioProperties): Boolean;
//------------------------------------------------------------------------------
// "ParseSingleMPEGAudioHeader" parses a single MPEG Audio Header      => END <=
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
// "GetMPEGAudioPropertyString"
// Returns in only one String (f_InfoString) Informations Header
// Header) about a MPEG File for use with scrolltexts or other...    => BEGIN <=
//------------------------------------------------------------------------------
function GetMPEGAudioPropertyString(f_Filename: string;
  f_Flags: TMPEGAudioPropertyConfig; out f_InfoString: string): Boolean;
//------------------------------------------------------------------------------
// "GetMPEGAudioPropertyString"
// Returns in only one String (f_InfoString) Informations Header
// Header) about a MPEG File for use with scrolltexts or other...      => END <=
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////

implementation


function IsValidMPEGAudioHeader(
  f_MPEGAudioBinaryFrameHeader: TBTMPEGAudioBinaryFrameHeader): Boolean;
begin
  Result := False;
  (* check for first part of Sync. (Byte1) *)
  if ((f_MPEGAudioBinaryFrameHeader[1] and $FF) <> $FF) then
    Exit;
  (* check for second part of Sync. (Byte2) *)
  if ((f_MPEGAudioBinaryFrameHeader[2] and $E0) <> $E0) then
    Exit;
  (* check for invalid Mpeg version (Byte2) *)
  if ((f_MPEGAudioBinaryFrameHeader[2] and $18) = $08) then
    Exit;
  (* check for invalid layer (Byte2) *)
  if ((f_MPEGAudioBinaryFrameHeader[2] and $06) = $00) then
    Exit;
  (* check for invalid BitRate *)
  if ((f_MPEGAudioBinaryFrameHeader[3] and $F0) = $F0) then
    Exit;
  (* check for invalid Sample rate (Byte3) *)
  if ((f_MPEGAudioBinaryFrameHeader[3] and $0C) = $0C) then
    Exit;
  (* check for invalid Evphasis (Byte4) *)
  if ((f_MPEGAudioBinaryFrameHeader[4] and $03) = $02) then
    Exit;
  (* .....more to come ?.....
     Header seems to be valid *)
  Result := True;
end;

function ParseSingleMPEGAudioHeader(
  f_MPEGAudioBinaryFrameHeader: TBTMPEGAudioBinaryFrameHeader;
  out f_MPEGAudioProperties: TBTMPEGAudioProperties): Boolean;
var
//  BitString: ShortString;
  FrameLengthMultiply: Integer;
begin
  Result := False;
  try
(*
{$IFDEF LogEvents}
    // output Header structure definition
    RaiseLogEvent('AAAAAAAAAAABBCCDEEEEFFGHIIJJKLMM');
    // get Header as Bitstring
    ByteArrayToBitString(f_MPEGAudioBinaryFrameHeader, 4, BitString);
    RaiseLogEvent(BitString);
{$ENDIF}
*)
    // get MPEG Version
    f_MPEGAudioProperties.Version :=
      MPEGAudioVersion[(f_MPEGAudioBinaryFrameHeader[2] and $18) shr 3];
    // get Audio Layer
    f_MPEGAudioProperties.Layer :=
      MPEGAudioLayer[(f_MPEGAudioBinaryFrameHeader[2] and $6) shr 1];
    // get CRC info
    f_MPEGAudioProperties.CRCProtected :=
      MPEGAudioCRCProtected[f_MPEGAudioBinaryFrameHeader[2] and $1];
    // get Bitrate
    f_MPEGAudioProperties.Bitrate :=
      MPEGAudioBitRates[f_MPEGAudioProperties.Version,
      f_MPEGAudioProperties.Layer,
      (f_MPEGAudioBinaryFrameHeader[3] and $F0) shr 4];
    // get Samplerate
     f_MPEGAudioProperties.SampleRate :=
      MPEGAudioSampleRates[f_MPEGAudioProperties.Version,
      (f_MPEGAudioBinaryFrameHeader[3] and $C) shr 2];
    // get padding bit
    f_MPEGAudioProperties.Padding :=
      Boolean((f_MPEGAudioBinaryFrameHeader[3] and $2) shr 1);
    // get private bit
    f_MPEGAudioProperties.PrivateBit :=
      Boolean(f_MPEGAudioBinaryFrameHeader[3] and $1);
    // get channel mode
    f_MPEGAudioProperties.CannelMode :=
      Byte((f_MPEGAudioBinaryFrameHeader[4] and $C0) shr 6);
    // modeextension is only with joint stereo available
    if f_MPEGAudioProperties.CannelMode =
      MPEGAudioChannelModeJointStereo then begin
      // get mode extension bits from frame header
      if (f_MPEGAudioProperties.Layer = 1) or
         (f_MPEGAudioProperties.Layer = 2) then begin
        // get Layer I & II mode extension
        f_MPEGAudioProperties.LowBand :=
          MPEGAudioLayer12ModeEx[(f_MPEGAudioBinaryFrameHeader[4] and $30)
            shr 4];
        f_MPEGAudioProperties.HighBand := MPEGAudioLayer12ModeExHighBand;
      end;
      if (f_MPEGAudioProperties.Layer = 3) then begin
        // get Layer III mode extension
        case ((f_MPEGAudioBinaryFrameHeader[4] and $30) shr 4) of
          $00: begin
              f_MPEGAudioProperties.IntensityStereo := False;
              f_MPEGAudioProperties.MSStereo := False;
            end;
          $01: begin
              f_MPEGAudioProperties.IntensityStereo := True;
              f_MPEGAudioProperties.MSStereo := False;
            end;
          $02: begin
              f_MPEGAudioProperties.IntensityStereo := False;
              f_MPEGAudioProperties.MSStereo := True;
            end;
          $03: begin
              f_MPEGAudioProperties.IntensityStereo := True;
              f_MPEGAudioProperties.MSStereo := True;
            end;
        end;
      end;
    end;
    // Get copyright info
    f_MPEGAudioProperties.CopyRighted :=
      Boolean((f_MPEGAudioBinaryFrameHeader[4] and $08) shr 3);
    // Get original info
    f_MPEGAudioProperties.Original :=
      Boolean((f_MPEGAudioBinaryFrameHeader[4] and $04) shr 2);
    // Get original info
    f_MPEGAudioProperties.Emphasis :=
      f_MPEGAudioBinaryFrameHeader[4] and $03;
    // get FrameLengthMultiply for FrameLength calculation
    f_MPEGAudioProperties.FrameLength := -1;
    if f_MPEGAudioProperties.Version = 1 then
      if f_MPEGAudioProperties.Layer = 1 then
        FrameLengthMultiply := 48000;
    if (f_MPEGAudioProperties.Layer = 2) or
       (f_MPEGAudioProperties.Layer = 3) then
      FrameLengthMultiply := 144000;
    if f_MPEGAudioProperties.Version = 2 then begin
      if f_MPEGAudioProperties.Layer = 1 then
        FrameLengthMultiply := 24000;
      if (f_MPEGAudioProperties.Layer = 2) or
         (f_MPEGAudioProperties.Layer = 3) then
        FrameLengthMultiply := 72000;
    end;
    // calculate Frame Length
    f_MPEGAudioProperties.FrameLength := FrameLengthMultiply *
      f_MPEGAudioProperties.Bitrate div f_MPEGAudioProperties.Samplerate +
      Integer(f_MPEGAudioProperties.Padding);
    if f_MPEGAudioProperties.FrameLength <= 0 then
      Abort;
    Result := True;
  except
{$IFDEF LogEvents}
    RaiseLogEvent('ParseSingleMPEGAudioHeader: CRITICAL ERROR: ' +
      'ERROR DUTING A IMPORTANT HEADER BASED CALCULATION!');
{$ENDIF}
  end;
end;

function ParseMPEGAudioFile(f_Filename: string; f_ParserConfig: TMPEGParseConfig;
  out f_MPEGAudioProperties: TBTMPEGAudioProperties): Boolean;
var
  FileStream: TFileStream;
 // WriteStream: TFileStream;
  Header: array[1..4] of byte;
  FirstHeaderFound: Boolean;
  ID3v1xTAG: TID3v1xTAGStrings;
  UnusedRemainingBytes: Integer;
//  Payload: array of byte;
  BitrateSum: Int64;
  BitRateTemp: Integer;
  VBR: Boolean;
{$IFDEF LogEvents}
  ErrLogCount: Integer;
{$ENDIF}
begin
  Result := False;
{$IFDEF LogEvents}
  ErrLogCount := 0;
{$ENDIF}
  BitrateSum := 0;
  BitRateTemp := 0;
  VBR := False;
  FirstHeaderFound := False;
  if not FileExists(f_Filename) then
    exit;
  // Check if a File contains ID3 Data
  if GetID3v1xTag(f_Filename, ID3v1xTAG) then begin
    UnusedRemainingBytes := SizeOf(TID3v1TAG);
{$IFDEF LogEvents}
    RaiseLogEvent('File Contains a ' + IntToStr(UnusedRemainingBytes) +
     ' Byte long ID3 Tag');
{$ENDIF}
  end else
    UnusedRemainingBytes := 0;
  FileStream := TFileStream.Create(f_Filename, fmOpenRead or fmShareDenyNone);
  FileStream.Position := 0;
//  WriteStream := TFileStream.Create(f_Filename + '.fix', fmCreate or fmShareDenyNone);
//  WriteStream.Position := 0;
{$IFDEF FileActionEvents}
  RaiseMinChangedEvent(0);
  RaiseMaxChangedEvent(Integer(FileStream.Size));
{$ENDIF}
  // Zero the Mem of the out record => just for confidence
  FillChar(f_MPEGAudioProperties, sizeof(TBTMPEGAudioProperties), $00);
  // set the Filesize to the out record
  f_MPEGAudioProperties.FileSize := FileStream.Size;
  while not ((FileStream.Position + 4) >= FileStream.Size - UnusedRemainingBytes) do begin
    FileStream.Read(Header[1], 4);
    // PreCheck if data might be a MPEG Audio Header
    if IsValidMPEGAudioHeader(TBTMPEGAudioBinaryFrameHeader(Header)) then begin
      try
        // ParseSingleMPEGAudioHeader checks for more errors (calculations....)
        if not ParseSingleMPEGAudioHeader(
          TBTMPEGAudioBinaryFrameHeader(Header), f_MPEGAudioProperties) then
          // Header is corrupt => skip Header
          Abort
        else
          Result := True;
//        WriteStream.Write(Header[1], 4);
        // Read Offset of first found header
        if not FirstHeaderFound then begin
           f_MPEGAudioProperties.FrameOffset := Integer(FileStream.Position -
             SizeOf(Header));
           FirstHeaderFound := True;
{$IFDEF LogEvents}
           RaiseLogEvent(IntToStr(f_MPEGAudioProperties.FrameOffset) +
             ' Bytes of garbage have been found on file start');
{$ENDIF}
        end;
        // add Bitrate to BitrateSum...
        BitrateSum := BitrateSum + f_MPEGAudioProperties.Bitrate;
        // Check for VBR....
        if (BitRateTemp <> 0) and (BitRateTemp <> f_MPEGAudioProperties.Bitrate) then
          // File seems to be a VBR file
          VBR := True;
        BitRateTemp := f_MPEGAudioProperties.Bitrate;
        // read and Write the Playload to file
//        SetLength(Payload, f_MPEGAudioProperties.FrameLength - 4);
//        FileStream.Read(Payload[0], f_MPEGAudioProperties.FrameLength - 4);
//        WriteStream.Write(Payload[0], f_MPEGAudioProperties.FrameLength - 4);
        (* jump one Frame forward in File
           - The FrameLength is defined as Header(4Byte) + Payload
           - The Payload size can be calculated:
             "Payloadsize := Framelength-Length(Header)"    *)
        FileStream.Position := FileStream.Position +
          f_MPEGAudioProperties.FrameLength - 4 ;
        if f_ParserConfig = ParseComplete_CBR_VBR then
          inc(f_MPEGAudioProperties.NbrOfFrames);
        if f_ParserConfig = ParseSingleFrameApprox_CBR then
           Break;
{$IFDEF FileActionEvents}
        RaisePosChangedEvent(Integer(FileStream.Position));
{$ENDIF}
      except
        FileStream.Position := FileStream.Position - 3;
{$IFDEF FileActionEvents}
        RaisePosChangedEvent(Integer(FileStream.Position));
{$ENDIF}
      end;
    end else begin
      FileStream.Position := FileStream.Position - 3;
{$IFDEF FileActionEvents}
        RaisePosChangedEvent(Integer(FileStream.Position));
{$ENDIF}
{$IFDEF LogEvents}
      if FirstHeaderFound then begin
{        RaiseLogEvent('Error in File at Byte ' + IntToStr(FileStream.Position) +
          ' => Corrput Header or garbage data inside the file....'); }
        inc(ErrLogCount);
      end;
{$ENDIF}
    end;
  end;
  // Full parse
  if f_ParserConfig = ParseComplete_CBR_VBR then begin
    // get average Bitrate
    if not (f_MPEGAudioProperties.NbrOfFrames <= 0) then begin
       f_MPEGAudioProperties.Bitrate :=
         Integer(Round(BitrateSum / f_MPEGAudioProperties.NbrOfFrames))
    end else begin
{$IFDEF LogEvents}
      RaiseLogEvent('WARNING: File Contains no MPEG Audio Frames!');
{$ENDIF}
    end;
         
    // set CBR/VBR
    f_MPEGAudioProperties.VariableBitRate := VBR;
    (* xy get song length "not shure if it works right with mpeg version 2"
          => a standard mp3 contains 38 frames per second.
             so 1 frame = 26/1000 of a second. *)
    f_MPEGAudioProperties.LengthInSeconds :=
      Integer(Round(f_MPEGAudioProperties.NbrOfFrames * 26 / 1000));
  end;
  // Single Frameparse approx
  if f_ParserConfig = ParseSingleFrameApprox_CBR then begin
    f_MPEGAudioProperties.NbrOfFrames := (FileStream.Size -
      f_MPEGAudioProperties.FrameOffset) div f_MPEGAudioProperties.FrameLength;
    (* xy get song length "not shure if it works right with mpeg version 2"
          => a standard mp3 contains 38 frames per second.
             so 1 frame = 26/1000 of a second. *)
    f_MPEGAudioProperties.LengthInSeconds :=
      Integer(Round(f_MPEGAudioProperties.NbrOfFrames * 26 / 1000));
  end;
{$IFDEF FileActionEvents}
        RaisePosChangedEvent(Integer(FileStream.Size));
{$ENDIF}
  if FileStream <> nil then
    FileStream.Free;
//  if WriteStream <> nil then
//    WriteStream.Free;
{$IFDEF LogEvents}
  RaiseLogEvent(IntToStr(ErrLogCount) +
    ' bytes of errors and/or garbage have been found since the first audio frame offset');
  if Result = True then
    RaiseLogEvent('File successfully parsed');
{$ENDIF}
end;

function GetMPEGAudioPropertyString(f_Filename: string;
  f_Flags: TMPEGAudioPropertyConfig; out f_InfoString: string): Boolean;
var
  ID3v1xTAG: TID3v1xTAGStrings;
  FrameHeader: TBTMPEGAudioProperties;
  FrameHeaderStrings: TBTMPEGAudioPropertyStrings;
begin
  Result := false;
  f_InfoString := EmptyStr;
  // clean up memory (just for confidence..)
  FillChar(ID3v1xTAG,sizeof(TID3v1xTAGStrings), $00);
  (***  READ ID3 TAG ***)
  if not GetID3v1xTag(f_Filename, ID3v1xTAG) then
    exit;
  // set Artist
  if m3iArtist in f_Flags then begin
    if ID3v1xTAG.Artist = EmptyStr then
      f_InfoString := f_InfoString + 'unknown artist'
    else
      f_InfoString := f_InfoString + ID3v1xTAG.Artist;
  end;
  // set Title
  if (m3iArtist in f_Flags) and (m3iTitle in f_Flags) then begin
    if ID3v1xTAG.Title = EmptyStr then
      f_InfoString := f_InfoString + ' - ' + 'unknown title' + '        '
    else
      f_InfoString := f_InfoString + ' - ' + ID3v1xTAG.Title + '        ';
  end;
  if (not (m3iArtist in f_Flags)) and (m3iTitle in f_Flags) then begin
    if ID3v1xTAG.Title = EmptyStr then
      f_InfoString := f_InfoString + 'unknown title' + '        '
    else
      f_InfoString := f_InfoString + ID3v1xTAG.Title + '        ';
  end;
  // set Track
  if (m3iTrack in f_Flags) and (ID3v1xTAG.Track <> EmptyStr) then
    f_InfoString := f_InfoString + 'Track ' + ID3v1xTAG.Track;
  // set Album
  if (m3iAlbum in f_Flags) and (ID3v1xTAG.Album <> EmptyStr) then
    f_InfoString := f_InfoString + ' from Album ' + ID3v1xTAG.Album;
  // set Year
  if (m3iYear in f_Flags) and (ID3v1xTAG.Year <> EmptyStr) then
    f_InfoString := f_InfoString + '        Released in ' + ID3v1xTAG.Year;
  // set genre
  if (m3iGenre in f_Flags) and (ID3v1xTAG.Genre <> EmptyStr) then
    f_InfoString := f_InfoString + '        Genre: ' + ID3v1xTAG.Genre;
  // set Comment
  if (m3iComment in f_Flags) and (ID3v1xTAG.Comment <> EmptyStr) then
    f_InfoString := f_InfoString + '        ' + ID3v1xTAG.Comment;
  (***  xy todo: READ only FIRST VALID MP3 HEADER ***)
  if not ParseMPEGAudioFile(f_Filename, ParseComplete_CBR_VBR, FrameHeader) then
    exit;
  // get as TMP3FrameHeaderInfoStrings
  FrameHeaderStrings := BTMPEGAudPropToMPEGAudPropStr(FrameHeader);
  if (m3iVersion in f_Flags) and (FrameHeaderStrings.Version <> EmptyStr) then
    f_InfoString := f_InfoString + '        ' + FrameHeaderStrings.Version;
  if (m3iLayer in f_Flags) and (FrameHeaderStrings.Layer <> EmptyStr) then
    f_InfoString := f_InfoString + '        ' + FrameHeaderStrings.Layer;
  if (m3iBitrate in f_Flags) and (FrameHeaderStrings.Bitrate <> EmptyStr) then
    f_InfoString := f_InfoString + '        Bitrate: ' + FrameHeaderStrings.Bitrate;
  if (m3iSamplerate in f_Flags) and (FrameHeaderStrings.Samplerate <> EmptyStr) then
    f_InfoString := f_InfoString + '        Samplerate: ' + FrameHeaderStrings.Samplerate;
  if (m3iCannelMode in f_Flags) and (FrameHeaderStrings.CannelMode <> EmptyStr) then
    f_InfoString := f_InfoString + '        Channelmode: ' + FrameHeaderStrings.CannelMode;
  Result := True;
end;

end.

